from typing import TypeAlias, Union

from codetocad.core import Dimension, Angle, Point, Dimensions
from codetocad.enums import Axis, LengthUnit, PresetLandmark, PresetMaterial

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from codetocad.interfaces import (
        MaterialInterface,
        PartInterface,
        EntityInterface,
        SketchInterface,
        LandmarkInterface,
        CameraInterface,
        VertexInterface,
        ExportableInterface,
    )

MaterialOrItsName: TypeAlias = Union[str, PresetMaterial, "MaterialInterface"]
PartOrItsName: TypeAlias = Union[str, "PartInterface"]
EntityOrItsName: TypeAlias = Union[str, "EntityInterface"]
SketchOrItsName: TypeAlias = Union[str, "SketchInterface"]
LandmarkOrItsName: TypeAlias = Union[str, "LandmarkInterface"]
CameraOrItsName: TypeAlias = Union[str, "CameraInterface"]
ExportableOrItsName: TypeAlias = Union[str, "ExportableInterface"]


FloatOrItsStringValue: TypeAlias = Union[str, float]
IntOrFloat: TypeAlias = Union[int, float]

AxisOrItsIndexOrItsName: TypeAlias = Union[str, int, Axis]
DimensionOrItsFloatOrStringValue: TypeAlias = Union[str, float, Dimension]
AngleOrItsFloatOrStringValue: TypeAlias = Union[str, float, Angle]
DimensionsOrItsListOfFloatOrString: TypeAlias = Union[
    str, list[FloatOrItsStringValue], list[Dimension], Dimensions
]
PointOrListOfFloatOrItsStringValue: TypeAlias = Union[
    str, list[FloatOrItsStringValue], list[Dimension], Point
]
PointOrListOfFloatOrItsStringValueOrVertex: TypeAlias = Union[
    str, list[FloatOrItsStringValue], list[Dimension], Point, "VertexInterface"
]
LengthUnitOrItsName: TypeAlias = Union[str, LengthUnit]
PresetLandmarkOrItsName: TypeAlias = Union[str, PresetLandmark]



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


class ExportableInterface(metaclass=ABCMeta):
    """An enttiy that can be exported."""

    @abstractmethod
    def export(self, file_path: str, overwrite: bool = True, scale: float = 1.0):
        """
        Export Entity. Use the filePath to control the export type, e.g. '/path/to/cube.obj' or '/path/to/curve.svg'
        """

        print("export is called in an abstract method. Please override this method.")
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import PartInterface


class MaterialInterface(metaclass=ABCMeta):
    """Materials affect the appearance and simulation properties of the parts."""

    name: str
    description: Optional[str] = None

    @abstractmethod
    def __init__(self, name: str, description: Optional[str] = None):
        self.name = name
        self.description = description

    @staticmethod
    def get_preset(parameter: "PresetMaterialInterface") -> "MaterialInterface":
        """
        Get a material from a preset
        """

        print(
            "get_preset is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def assign_to_part(self, part_name_or_instance: PartOrItsName):
        """
        Assigns the material to a part.
        """

        print(
            "assign_to_part is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_color(
        self,
        r_value: IntOrFloat,
        g_value: IntOrFloat,
        b_value: IntOrFloat,
        a_value: IntOrFloat = 1.0,
    ):
        """
        Set the RGBA color of an entity. Supports 0-255 int or 0.0-1.0 float values.
        """

        print("set_color is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def set_reflectivity(self, reflectivity: float):
        """
        Change the surface reflectivity (metallic luster) of the material.
        """

        print(
            "set_reflectivity is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_roughness(self, roughness: float):
        """
        Change the surface roughness of the material.
        """

        print(
            "set_roughness is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_image_texture(self, image_file_path: str):
        """
        Add a texture from an image file.
        """

        print(
            "set_image_texture is called in an abstract method. Please override this method."
        )
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class MirrorableInterface(metaclass=ABCMeta):
    """An entity that can be mirrored."""

    @abstractmethod
    def mirror(
        self,
        mirror_across_entity: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName,
        resulting_mirrored_entity_name: Optional[str] = None,
    ):
        """
        Mirror an existing entity with respect to a landmark. If a name is provided, the mirror becomes a separate entity.
        """

        print("mirror is called in an abstract method. Please override this method.")
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


class ImportableInterface(metaclass=ABCMeta):
    """An entity that can be imported from a file."""

    @abstractmethod
    def create_from_file(self, file_path: str, file_type: Optional[str] = None):
        """
        Imports geometry from a file.
        """

        print(
            "create_from_file is called in an abstract method. Please override this method."
        )
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import RenderInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Camera


class Render(RenderInterface):
    def render_image(
        self,
        output_file_path: str,
        overwrite: bool = True,
        file_type: Optional[str] = None,
    ):
        print("render_image called:", output_file_path, overwrite, file_type)
        return self

    def render_video_mp4(
        self,
        output_file_path: str,
        start_frame_number: "int" = 1,
        end_frame_number: "int" = 100,
        step_frames: "int" = 1,
        overwrite: bool = True,
    ):
        print(
            "render_video_mp4 called:",
            output_file_path,
            start_frame_number,
            end_frame_number,
            step_frames,
            overwrite,
        )
        return self

    def render_video_frames(
        self,
        output_folder_path: str,
        file_name_prefix: str,
        start_frame_number: "int" = 1,
        end_frame_number: "int" = 100,
        step_frames: "int" = 1,
        overwrite: bool = True,
        file_type: Optional[str] = None,
    ):
        print(
            "render_video_frames called:",
            output_folder_path,
            file_name_prefix,
            start_frame_number,
            end_frame_number,
            step_frames,
            overwrite,
            file_type,
        )
        return self

    def set_frame_rate(self, frame_rate: "int"):
        print("set_frame_rate called:", frame_rate)
        return self

    def set_resolution(self, x: "int", y: "int"):
        print("set_resolution called:", x, y)
        return self

    def set_render_quality(self, quality: "int"):
        print("set_render_quality called:", quality)
        return self

    def set_render_engine(self, name: str):
        print("set_render_engine called:", name)
        return self

    def set_camera(self, camera_name_or_instance: CameraOrItsName):
        print("set_camera called:", camera_name_or_instance)
        return self



from typing import Optional
from codetocad.enums.length_unit import LengthUnit


class BoundaryAxis:
    min: float
    max: float
    unit: Optional[LengthUnit]

    @property
    def center(self) -> float:
        return (self.max + self.min) / 2.0

    def __init__(self, min: float, max: float, unit=None) -> None:
        self.min = min
        self.max = max

        if unit is None:
            return

        unit = (
            LengthUnit.from_string(unit.replace(" ", "").lower())
            if isinstance(unit, str)
            else unit
        )
        assert isinstance(
            unit, LengthUnit
        ), "Dimension unit must be of type LengthUnit or string."

        self.unit = unit

    def __str__(self):
        return f"""    min   max   unit
x   {self.min}  {self.max}  {self.unit.name+'(s)' if self.unit else "No Unit"}
"""

    def __repr__(self) -> str:
        return self.__str__()



from .angle import Angle
from .boundary_axis import BoundaryAxis
from .boundary_box import BoundaryBox
from .dimension import Dimension
from .dimensions import Dimensions
from .point import Point
from .math_utils import *
from .shapes import *



from codetocad.enums.units import Units
from typing import Optional


class AngleUnit(Units):
    RADIANS = 0
    DEGREES = 1

    @staticmethod
    def from_string(name: str) -> "AngleUnit":
        aliases: dict[str, AngleUnit] = {
            "radians": AngleUnit.RADIANS,
            "rad": AngleUnit.RADIANS,
            "rads": AngleUnit.RADIANS,
            "r": AngleUnit.RADIANS,
            "degrees": AngleUnit.DEGREES,
            "degree": AngleUnit.DEGREES,
            "degs": AngleUnit.DEGREES,
            "deg": AngleUnit.DEGREES,
            "d": AngleUnit.DEGREES,
        }

        from_string: str = name.lower().replace("(s)", "")

        parsedUnit: Optional[AngleUnit] = (
            aliases[from_string] if from_string in aliases else None
        )

        assert parsedUnit is not None, f"Could not parse unit {from_string}"

        return parsedUnit



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import EntityInterface

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class LandmarkInterface(EntityInterface, metaclass=ABCMeta):
    """Landmarks are named positions on an entity."""

    name: str
    parent_entity: EntityOrItsName
    description: Optional[str] = None

    @abstractmethod
    def __init__(
        self,
        name: str,
        parent_entity: EntityOrItsName,
        description: Optional[str] = None,
        native_instance=None,
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.name = name
        self.parent_entity = parent_entity
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def clone(
        self,
        new_name: str,
        offset: Optional[DimensionsOrItsListOfFloatOrString] = None,
        new_parent: Optional[EntityOrItsName] = None,
    ) -> "LandmarkInterface":
        """
        Clone an existing Landmark with an optional offset, and reassignment to a different parent. Returns the new Landmark.
        """

        print("clone is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def get_landmark_entity_name(self) -> str:
        """
        Get the landmark object name in the scene, which may be different to the name of the landmark when it was first created. For example, the generated name may be {parentName}_{landmarkName}.
        """

        print(
            "get_landmark_entity_name is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def get_parent_entity(self) -> "EntityInterface":
        """
        Get the name of the entity this landmark belongs to.
        """

        print(
            "get_parent_entity is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()



from codetocad.enums.units import Units


class LengthUnit(Units):
    # metric
    nm = 1 / 1e6
    μm = 1 / 1e3
    mm = 1
    cm = 10
    m = 1000
    km = 1000000

    # imperial
    thou = 25.4 / 1000
    inch = 25.4
    ft = 25.4 * 12
    mi = 25.4 * 63360

    @staticmethod
    def from_string(name: str) -> "LengthUnit":
        aliases: dict[str, LengthUnit] = {
            # metric
            "nanometer": LengthUnit.nm,
            "nanometers": LengthUnit.nm,
            "micrometer": LengthUnit.μm,
            "micrometers": LengthUnit.μm,
            "micron": LengthUnit.μm,
            "microns": LengthUnit.μm,
            "millimeter": LengthUnit.mm,
            "millimeters": LengthUnit.mm,
            "centimeter": LengthUnit.cm,
            "centimeters": LengthUnit.cm,
            "kilometer": LengthUnit.km,
            "meter": LengthUnit.m,
            "meters": LengthUnit.m,
            "nm": LengthUnit.nm,
            "μm": LengthUnit.μm,
            "mm": LengthUnit.mm,
            "cm": LengthUnit.cm,
            "m": LengthUnit.m,
            "km": LengthUnit.km,
            # imperial
            "thousandthInch": LengthUnit.thou,
            "thousandth": LengthUnit.thou,
            "inch": LengthUnit.inch,
            "inches": LengthUnit.inch,
            "foot": LengthUnit.ft,
            "feet": LengthUnit.ft,
            "mile": LengthUnit.mi,
            "miles": LengthUnit.mi,
            "thou": LengthUnit.thou,
            "in": LengthUnit.inch,
            "ft": LengthUnit.ft,
            "mi": LengthUnit.mi,
        }

        from_string: str = name.lower().replace("(s)", "")

        parsedUnit: LengthUnit | None = (
            aliases[from_string] if from_string in aliases else None
        )

        assert parsedUnit is not None, f"Could not parse unit {from_string}"

        return parsedUnit



def linspace(start: float, stop: float, steps: int, endpoint=True):
    # References https://gist.github.com/pmav99/d124872c879f3e9fa51e
    if endpoint:
        step = (stop - start) / (steps - 1)
    else:
        step = (stop - start) / steps

    for i in range(steps):
        yield start + step * i



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import LandmarkInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Entity


class Landmark(Entity, LandmarkInterface):
    name: str
    parent_entity: EntityOrItsName
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self,
        name: str,
        parent_entity: EntityOrItsName,
        description: Optional[str] = None,
        native_instance=None,
    ):
        self.name = name
        self.parent_entity = parent_entity
        self.description = description
        self.native_instance = native_instance

    def clone(
        self,
        new_name: str,
        offset: Optional[DimensionsOrItsListOfFloatOrString] = None,
        new_parent: Optional[EntityOrItsName] = None,
    ) -> "Landmark":
        print("clone called:", new_name, offset, new_parent)
        from . import Landmark

        return Landmark("name", "parent")

    def get_landmark_entity_name(self) -> str:
        print(
            "get_landmark_entity_name called:",
        )
        return "String"

    def get_parent_entity(self) -> "Entity":
        print(
            "get_parent_entity called:",
        )
        from . import Entity

        return Entity("an entity")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class AnimationInterface(metaclass=ABCMeta):
    """Animation related functionality."""

    @abstractmethod
    def __init__(self):
        pass

    @staticmethod
    def default() -> "AnimationInterface":
        raise RuntimeError()

    @abstractmethod
    def set_frame_start(self, frame_number: "int"):
        """
        Set the start animation frame in the scene.
        """

        print(
            "set_frame_start is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_frame_end(self, frame_number: "int"):
        """
        Set the end animation frame in the scene.
        """

        print(
            "set_frame_end is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_frame_current(self, frame_number: "int"):
        """
        Set the current animation frame in the scene.
        """

        print(
            "set_frame_current is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_key_frame_location(self, entity: EntityOrItsName, frame_number: "int"):
        """
        Create an animation key-frame using the location of the entity.
        """

        print(
            "create_key_frame_location is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_key_frame_rotation(self, entity: EntityOrItsName, frame_number: "int"):
        """
        Create an animation key-frame using the rotation of the entity.
        """

        print(
            "create_key_frame_rotation is called in an abstract method. Please override this method."
        )
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import LightInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity


class Light(Entity, LightInterface):
    name: str
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self, name: str, description: Optional[str] = None, native_instance=None
    ):
        self.name = name
        self.description = description
        self.native_instance = native_instance

    def set_color(self, r_value: IntOrFloat, g_value: IntOrFloat, b_value: IntOrFloat):
        print("set_color called:", r_value, g_value, b_value)
        return self

    def create_sun(self, energy_level: float):
        print("create_sun called:", energy_level)
        return self

    def create_spot(self, energy_level: float):
        print("create_spot called:", energy_level)
        return self

    def create_point(self, energy_level: float):
        print("create_point called:", energy_level)
        return self

    def create_area(self, energy_level: float):
        print("create_area called:", energy_level)
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from codetocad.interfaces import (
    MirrorableInterface,
    PatternableInterface,
    ImportableInterface,
    ExportableInterface,
    ScalableInterface,
)

from . import EntityInterface

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import PartInterface
    from . import EntityInterface
    from . import WireInterface
    from . import VertexInterface
    from . import EdgeInterface


class SketchInterface(
    EntityInterface,
    MirrorableInterface,
    PatternableInterface,
    ImportableInterface,
    ExportableInterface,
    ScalableInterface,
    metaclass=ABCMeta,
):
    """Capabilities related to creating and manipulating 2D sketches, composed of vertices, edges and wires."""

    name: str
    curve_type: Optional["CurveTypes"] = None
    description: Optional[str] = None

    @abstractmethod
    def __init__(
        self,
        name: str,
        curve_type: Optional["CurveTypes"] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.name = name
        self.curve_type = curve_type
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def clone(self, new_name: str, copy_landmarks: bool = True) -> "SketchInterface":
        """
        Clone an existing sketch with its geometry and properties. Returns the new Sketch.
        """

        print("clone is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def revolve(
        self,
        angle: AngleOrItsFloatOrStringValue,
        about_entity_or_landmark: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName = "z",
    ) -> "PartInterface":
        """
        Revolve a Sketch around another Entity or Landmark
        """

        print("revolve is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def twist(
        self,
        angle: AngleOrItsFloatOrStringValue,
        screw_pitch: DimensionOrItsFloatOrStringValue,
        iterations: "int" = 1,
        axis: AxisOrItsIndexOrItsName = "z",
    ):
        """
        AKA Helix, Screw.
        """

        print("twist is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def extrude(self, length: DimensionOrItsFloatOrStringValue) -> "PartInterface":
        """
        Extrude a curve by a specified length. Returns a Part type.
        """

        print("extrude is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def sweep(
        self, profile_name_or_instance: SketchOrItsName, fill_cap: bool = True
    ) -> "PartInterface":
        """
        Extrude this Sketch along the path of another Sketch
        """

        print("sweep is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def offset(self, radius: DimensionOrItsFloatOrStringValue):
        """
        Uniformly add a wall around a Sketch.
        """

        print("offset is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def profile(self, profile_curve_name: str):
        """
        Bend this curve along the path of another
        """

        print("profile is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def create_text(
        self,
        text: str,
        font_size: DimensionOrItsFloatOrStringValue = 1.0,
        bold: bool = False,
        italic: bool = False,
        underlined: bool = False,
        character_spacing: "int" = 1,
        word_spacing: "int" = 1,
        line_spacing: "int" = 1,
        font_file_path: Optional[str] = None,
    ):
        """
        Adds text to a sketch.
        """

        print(
            "create_text is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_from_vertices(
        self, points: list[PointOrListOfFloatOrItsStringValueOrVertex]
    ) -> "WireInterface":
        """
        Create a curve from 2D/3D points.
        """

        print(
            "create_from_vertices is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_point(
        self, point: PointOrListOfFloatOrItsStringValue
    ) -> "VertexInterface":
        """
        Create a point
        """

        print(
            "create_point is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_line(
        self,
        start_at: PointOrListOfFloatOrItsStringValueOrVertex,
        end_at: PointOrListOfFloatOrItsStringValueOrVertex,
    ) -> "EdgeInterface":
        """
        Create a line between two points
        """

        print(
            "create_line is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_circle(
        self, radius: DimensionOrItsFloatOrStringValue
    ) -> "WireInterface":
        """
        Create a circle
        """

        print(
            "create_circle is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_ellipse(
        self,
        radius_minor: DimensionOrItsFloatOrStringValue,
        radius_major: DimensionOrItsFloatOrStringValue,
    ) -> "WireInterface":
        """
        Create an ellipse
        """

        print(
            "create_ellipse is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_arc(
        self,
        start_at: PointOrListOfFloatOrItsStringValueOrVertex,
        end_at: PointOrListOfFloatOrItsStringValueOrVertex,
        radius: DimensionOrItsFloatOrStringValue,
        flip: Optional[bool] = False,
    ) -> "WireInterface":
        """
        Create an arc. The radius is the distance from the center of the circle that forms the arc, to the chord tying start_at and end_at.
        """

        print(
            "create_arc is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_rectangle(
        self,
        length: DimensionOrItsFloatOrStringValue,
        width: DimensionOrItsFloatOrStringValue,
    ) -> "WireInterface":
        """
        Create a rectangle
        """

        print(
            "create_rectangle is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_polygon(
        self,
        number_of_sides: "int",
        length: DimensionOrItsFloatOrStringValue,
        width: DimensionOrItsFloatOrStringValue,
    ) -> "WireInterface":
        """
        Create an n-gon
        """

        print(
            "create_polygon is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_trapezoid(
        self,
        length_upper: DimensionOrItsFloatOrStringValue,
        length_lower: DimensionOrItsFloatOrStringValue,
        height: DimensionOrItsFloatOrStringValue,
    ) -> "WireInterface":
        """
        Create a trapezoid
        """

        print(
            "create_trapezoid is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def create_spiral(
        self,
        number_of_turns: "int",
        height: DimensionOrItsFloatOrStringValue,
        radius: DimensionOrItsFloatOrStringValue,
        is_clockwise: bool = True,
        radius_end: Optional[DimensionOrItsFloatOrStringValue] = None,
    ) -> "WireInterface":
        """
        Create a spiral or helix
        """

        print(
            "create_spiral is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class AnalyticsInterface(metaclass=ABCMeta):
    """Tools for collecting data about the entities and scene."""

    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def measure_distance(
        self, entity1: EntityOrItsName, entity2: EntityOrItsName
    ) -> "Dimensions":
        """
        The ubiquitous ruler.
        """

        print(
            "measure_distance is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def measure_angle(
        self,
        entity1: EntityOrItsName,
        entity2: EntityOrItsName,
        pivot: Optional[EntityOrItsName] = None,
    ) -> "list[Angle]":
        """
        The ubiquitous ruler.
        """

        print(
            "measure_angle is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def get_world_pose(self, entity: EntityOrItsName) -> "list[float]":
        """
        Returns the world pose of an entity.
        """

        print(
            "get_world_pose is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def get_bounding_box(self, entity_name: EntityOrItsName) -> "BoundaryBox":
        """
        Returns the bounding box of an entity.
        """

        print(
            "get_bounding_box is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def get_dimensions(self, entity_name: EntityOrItsName) -> "Dimensions":
        """
        Returns the dimensions of an entity.
        """

        print(
            "get_dimensions is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def log(self, message: str):
        """
        Write a message
        """

        print("log is called in an abstract method. Please override this method.")
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class PatternableInterface(metaclass=ABCMeta):
    """An entity that can be patterned."""

    @abstractmethod
    def linear_pattern(
        self,
        instance_count: "int",
        offset: DimensionOrItsFloatOrStringValue,
        direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        """
        Pattern in a uniform direction.
        """

        print(
            "linear_pattern is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: AngleOrItsFloatOrStringValue,
        center_entity_or_landmark: EntityOrItsName,
        normal_direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        """
        Pattern in a circular direction.
        """

        print(
            "circular_pattern is called in an abstract method. Please override this method."
        )
        return self



import unittest


class TestProviderCompliance(unittest.TestCase):
    def test_blender_provider(self):
        # NOTE: if you're running this test locally, you might need `pip install fake-bpy-module-latest` or Blender API added to python path
        from providers.blender.blender_provider import Part, Sketch, Landmark

        Part("")
        Sketch("")
        Landmark("", "")



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import WireInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Edge
    from . import Entity
    from . import Vertex
    from . import Part


class Wire(Entity, WireInterface):
    def mirror(
        self,
        mirror_across_entity: EntityOrItsName,
        axis: AxisOrItsIndexOrItsName,
        resulting_mirrored_entity_name: Optional[str] = None,
    ):
        print(
            "mirror called:", mirror_across_entity, axis, resulting_mirrored_entity_name
        )
        return self

    def linear_pattern(
        self,
        instance_count: "int",
        offset: DimensionOrItsFloatOrStringValue,
        direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print("linear_pattern called:", instance_count, offset, direction_axis)
        return self

    def circular_pattern(
        self,
        instance_count: "int",
        separation_angle: AngleOrItsFloatOrStringValue,
        center_entity_or_landmark: EntityOrItsName,
        normal_direction_axis: AxisOrItsIndexOrItsName = "z",
    ):
        print(
            "circular_pattern called:",
            instance_count,
            separation_angle,
            center_entity_or_landmark,
            normal_direction_axis,
        )
        return self

    def project(self, project_onto: "Sketch") -> "Projectable":
        print("project called:", project_onto)
        from . import Sketch

        return Sketch("a projected sketch")

    edges: "list[Edge]"
    parent_entity: Optional[EntityOrItsName] = None
    name: str
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self,
        edges: "list[Edge]",
        name: str,
        parent_entity: Optional[EntityOrItsName] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        self.edges = edges
        self.parent_entity = parent_entity
        self.name = name
        self.description = description
        self.native_instance = native_instance

    def get_vertices(self) -> "list[Vertex]":
        print(
            "get_vertices called:",
        )
        from . import Vertex

        return [Vertex(Point.from_list_of_float_or_string([0, 0, 0]), "a vertex")]

    def is_closed(self) -> bool:
        print(
            "is_closed called:",
        )
        return True

    def loft(self, other: "Wire", new_part_name: Optional[str] = None) -> "Part":
        print("loft called:", other, new_part_name)
        from . import Part

        return Part("a part")



from codetocad.utilities import *
from codetocad.codetocad_types import *
from codetocad.interfaces import *

from importlib.util import find_spec

if find_spec("blender_provider"):
    # this will only be available when the Blender Addon is packaged:
    from blender_provider import *
else:
    # When this python library is bundled, use the providers_sample if the "providers" directory is not copied.
    from codetocad.providers_sample import *



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


class SubdividableInterface(metaclass=ABCMeta):
    """An entity that can be broken down or scaled up into more components."""

    @abstractmethod
    def remesh(self, strategy: str, amount: float):
        """
        Remeshing changes the shape of an entity.
        """

        print("remesh is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def subdivide(self, amount: float):
        """
        Subdivide an entity into more components.
        """

        print("subdivide is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def decimate(self, amount: float):
        """
        Decimate an entity into less components.
        """

        print("decimate is called in an abstract method. Please override this method.")
        return self



# THIS IS AN AUTO-GENERATED FILE. DO NOT CHANGE.

from .scalable_interface import ScalableInterface
from .mirrorable_interface import MirrorableInterface
from .patternable_interface import PatternableInterface
from .subdividable_interface import SubdividableInterface
from .importable_interface import ImportableInterface
from .exportable_interface import ExportableInterface
from .projectable_interface import ProjectableInterface
from .entity_interface import EntityInterface
from .part_interface import PartInterface
from .sketch_interface import SketchInterface
from .vertex_interface import VertexInterface
from .edge_interface import EdgeInterface
from .wire_interface import WireInterface
from .landmark_interface import LandmarkInterface
from .joint_interface import JointInterface
from .material_interface import MaterialInterface
from .animation_interface import AnimationInterface
from .light_interface import LightInterface
from .camera_interface import CameraInterface
from .render_interface import RenderInterface
from .scene_interface import SceneInterface
from .analytics_interface import AnalyticsInterface



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import JointInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Entity


class Joint(JointInterface):
    entity1: EntityOrItsName
    entity2: EntityOrItsName

    def __init__(self, entity1: EntityOrItsName, entity2: EntityOrItsName):
        self.entity1 = entity1
        self.entity2 = entity2

    def translate_landmark_onto_another(self):
        print(
            "translate_landmark_onto_another called:",
        )
        return self

    def pivot(self):
        print(
            "pivot called:",
        )
        return self

    def gear_ratio(self, ratio: float):
        print("gear_ratio called:", ratio)
        return self

    def limit_location_xyz(
        self,
        x: Optional[DimensionOrItsFloatOrStringValue] = None,
        y: Optional[DimensionOrItsFloatOrStringValue] = None,
        z: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        print("limit_location_xyz called:", x, y, z)
        return self

    def limit_location_x(
        self,
        min: Optional[DimensionOrItsFloatOrStringValue] = None,
        max: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        print("limit_location_x called:", min, max)
        return self

    def limit_location_y(
        self,
        min: Optional[DimensionOrItsFloatOrStringValue] = None,
        max: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        print("limit_location_y called:", min, max)
        return self

    def limit_location_z(
        self,
        min: Optional[DimensionOrItsFloatOrStringValue] = None,
        max: Optional[DimensionOrItsFloatOrStringValue] = None,
    ):
        print("limit_location_z called:", min, max)
        return self

    def limit_rotation_xyz(
        self,
        x: Optional[AngleOrItsFloatOrStringValue] = None,
        y: Optional[AngleOrItsFloatOrStringValue] = None,
        z: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        print("limit_rotation_xyz called:", x, y, z)
        return self

    def limit_rotation_x(
        self,
        min: Optional[AngleOrItsFloatOrStringValue] = None,
        max: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        print("limit_rotation_x called:", min, max)
        return self

    def limit_rotation_y(
        self,
        min: Optional[AngleOrItsFloatOrStringValue] = None,
        max: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        print("limit_rotation_y called:", min, max)
        return self

    def limit_rotation_z(
        self,
        min: Optional[AngleOrItsFloatOrStringValue] = None,
        max: Optional[AngleOrItsFloatOrStringValue] = None,
    ):
        print("limit_rotation_z called:", min, max)
        return self



from enum import Enum


class CurvePrimitiveTypes(Enum):
    Point = 0
    LineTo = 1
    Line = 2
    Angle = 3
    Circle = 4
    Ellipse = 5
    Sector = 6
    Segment = 7
    Rectangle = 8
    Rhomb = 9
    Trapezoid = 10
    Polygon = 11
    Polygon_ab = 12
    Arc = 13
    Spiral = 14



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface
    from . import ExportableInterface


class SceneInterface(metaclass=ABCMeta):
    """Scene, camera, lighting, rendering, animation, simulation and GUI related functionality."""

    name: Optional[str] = None
    description: Optional[str] = None

    @abstractmethod
    def __init__(self, name: Optional[str] = None, description: Optional[str] = None):
        self.name = name
        self.description = description

    @staticmethod
    def default() -> "SceneInterface":
        raise RuntimeError()

    @abstractmethod
    def create(self):
        """
        Creates a new scene.
        """

        print("create is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def delete(self):
        """
        Deletes a scene.
        """

        print("delete is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def is_exists(self) -> bool:
        """
        Check if the scene exists
        """

        print("is_exists is called in an abstract method. Please override this method.")
        raise NotImplementedError()

    @abstractmethod
    def get_selected_entity(self) -> "EntityInterface":
        """
        Get the selected entity in the Scene.
        """

        print(
            "get_selected_entity is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()

    @abstractmethod
    def export(
        self,
        file_path: str,
        entities: "list[string,ExportableInterface]",
        overwrite: bool = True,
        scale: float = 1.0,
    ):
        """
        Export the entire scene or specific entities.
        """

        print("export is called in an abstract method. Please override this method.")
        return self

    @abstractmethod
    def set_default_unit(self, unit: LengthUnitOrItsName):
        """
        Set the document's default measurements system.
        """

        print(
            "set_default_unit is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def create_group(self, name: str):
        """
        Create a new group
        """

        print(
            "create_group is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def delete_group(self, name: str, remove_children: bool):
        """
        Delete a new group
        """

        print(
            "delete_group is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def remove_from_group(self, entity_name: str, group_name: str):
        """
        Removes an existing entity from a group
        """

        print(
            "remove_from_group is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def assign_to_group(
        self,
        entities: list[EntityOrItsName],
        group_name: str,
        remove_from_other_groups: Optional[bool] = True,
    ):
        """
        Assigns an existing entity to a new group
        """

        print(
            "assign_to_group is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_visible(self, entities: list[EntityOrItsName], is_visible: bool):
        """
        Change the visibiltiy of the entity.
        """

        print(
            "set_visible is called in an abstract method. Please override this method."
        )
        return self

    @abstractmethod
    def set_background_image(
        self,
        file_path: str,
        location_x: Optional[DimensionOrItsFloatOrStringValue] = 0,
        location_y: Optional[DimensionOrItsFloatOrStringValue] = 0,
    ):
        """
        Set the scene background image. This can be an image or an HDRI texture.
        """

        print(
            "set_background_image is called in an abstract method. Please override this method."
        )
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import AnimationInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Entity


class Animation(AnimationInterface):
    def __init__(self):
        pass

    @staticmethod
    def default() -> "Animation":
        return Animation()

    def set_frame_start(self, frame_number: "int"):
        print("set_frame_start called:", frame_number)
        return self

    def set_frame_end(self, frame_number: "int"):
        print("set_frame_end called:", frame_number)
        return self

    def set_frame_current(self, frame_number: "int"):
        print("set_frame_current called:", frame_number)
        return self

    def create_key_frame_location(self, entity: EntityOrItsName, frame_number: "int"):
        print("create_key_frame_location called:", entity, frame_number)
        return self

    def create_key_frame_rotation(self, entity: EntityOrItsName, frame_number: "int"):
        print("create_key_frame_rotation called:", entity, frame_number)
        return self



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import AnalyticsInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import Entity


class Analytics(AnalyticsInterface):
    def __init__(self):
        pass

    def measure_distance(
        self, entity1: EntityOrItsName, entity2: EntityOrItsName
    ) -> "Dimensions":
        print("measure_distance called:", entity1, entity2)
        return Dimensions.from_point(Point.from_list_of_float_or_string([0, 0, 0]))

    def measure_angle(
        self,
        entity1: EntityOrItsName,
        entity2: EntityOrItsName,
        pivot: Optional[EntityOrItsName] = None,
    ) -> "list[Angle]":
        print("measure_angle called:", entity1, entity2, pivot)
        return [Angle("90")]

    def get_world_pose(self, entity: EntityOrItsName) -> "list[float]":
        print("get_world_pose called:", entity)
        return [0.0]

    def get_bounding_box(self, entity_name: EntityOrItsName) -> "BoundaryBox":
        print("get_bounding_box called:", entity_name)
        return BoundaryBox(BoundaryAxis(0, 0), BoundaryAxis(0, 0), BoundaryAxis(0, 0))

    def get_dimensions(self, entity_name: EntityOrItsName) -> "Dimensions":
        print("get_dimensions called:", entity_name)
        return Dimensions.from_point(Point.from_list_of_float_or_string([0, 0, 0]))

    def log(self, message: str):
        print("log called:", message)
        return self



import subprocess
from setuptools import setup
from pathlib import Path

long_description = (Path(__file__).parent / "README.md").read_text(encoding="utf-8")

git_commit_epoch = 0

try:
    git_commit_epoch = (
        subprocess.check_output(["git", "show", "-s", "--format=%ct", "HEAD"])
        .strip()
        .decode()
    )
    open("version.txt", "w").write(git_commit_epoch)
except:
    git_commit_epoch = open("version.txt").read()

setup(
    name="codetocad",
    version=f"0.3.{git_commit_epoch}",
    description="3D modeling automation in your favorite modeling software.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/CodeToCAD/CodeToCAD",
    author="CodeToCAD",
    author_email="shehab@codethatdown.com",
    license="GPL v3",
    entry_points={
        "console_scripts": ["codetocad=codetocad.run:execute_launcher"],
    },
    packages=[
        "codetocad",
        "codetocad.interfaces",
        "codetocad.providers_sample",
        "codetocad.core",
        "codetocad.core.shapes",
        "codetocad.enums",
        "codetocad.launchers",
    ],
    install_requires=[],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import SketchInterface


class ProjectableInterface(metaclass=ABCMeta):
    """This entity can be projected onto a surface"""

    @abstractmethod
    def project(self, project_onto: "SketchInterface") -> "ProjectableInterface":
        """
        Project this entity onto another
        """

        print("project is called in an abstract method. Please override this method.")
        raise NotImplementedError()



from codetocad.launchers.blender import run_blender_process
from codetocad.launchers.launcher_args import LauncherArgs
from codetocad.launchers.unknown_launcher import run_unknown_process


def execute_launcher():
    args = LauncherArgs.from_cli_args()

    if args.is_sample_launcher():
        print("Running script with the provider_sample.")
        return

    launcher_lower = args.launcher.lower()

    if launcher_lower == "blender":
        return run_blender_process(args)

    run_unknown_process(args)


if __name__ == "__main__":
    execute_launcher()



from dataclasses import asdict, dataclass
import argparse
from typing import Optional

from codetocad.utilities import get_absolute_filepath


@dataclass
class LauncherArgs:
    script_file_path: str
    launcher: str
    launcher_location: Optional[str] = None
    background: Optional[bool] = None
    document_name: Optional[str] = None
    config_file_path: Optional[str] = None

    @staticmethod
    def get_sample_launcher_name():
        return "sample"

    def is_sample_launcher(self):
        return self.launcher == LauncherArgs.get_sample_launcher_name()

    def to_subprocess_args(self):
        args = asdict(
            self, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}
        )
        del args["launcher"]
        del args["launcher_location"]

        args_list = [self.launcher_location or self.launcher]

        for key, value in args.items():
            args_list.append(f"--{key}")
            args_list.append(str(value))

        return args_list

    @staticmethod
    def from_cli_args():
        parser = argparse.ArgumentParser(
            description="Welcome to the CodeToCAD cli tool. Visit https://github.com/CodeToCAD/CodeToCAD for help or information."
        )

        parser.add_argument(
            "script_file_path", type=str, help="CodeToCAD script to execute"
        )

        parser.add_argument(
            "launcher",
            type=str,
            nargs="?",
            const=LauncherArgs.get_sample_launcher_name(),
            help="name of the launcher to execute. You can specify a location using the --launcher_location argument.",
        )

        parser.add_argument(
            "--launcher_location", type=str, help="Path or URL of the launcher"
        )

        parser.add_argument(
            "--background",
            action="store_true",
            help="specify if the process should run in the background",
        )

        parser.add_argument(
            "--document_name",
            type=str,
            help="specify the document name or path to save the output file to",
        )

        parser.add_argument(
            "--config_file_path",
            type=str,
            help="specify the path to a config file to be read by the launcher",
        )

        args = parser.parse_args()

        return LauncherArgs(
            script_file_path=get_absolute_filepath(args.script_file_path, True),
            launcher=args.launcher,
            launcher_location=args.launcher_location,
            background=args.background,
            document_name=args.document_name,
            config_file_path=args.config_file_path,
        )



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.
# Copy this file and remove this header to create a new CodeToCAD Provider.

from typing import Optional

from codetocad.interfaces import CameraInterface

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from . import Entity


class Camera(Entity, CameraInterface):
    name: str
    description: Optional[str] = None
    native_instance = None

    def __init__(
        self, name: str, description: Optional[str] = None, native_instance=None
    ):
        self.name = name
        self.description = description
        self.native_instance = native_instance

    def create_perspective(self):
        print(
            "create_perspective called:",
        )
        return self

    def create_orthogonal(self):
        print(
            "create_orthogonal called:",
        )
        return self

    def set_focal_length(self, length: float):
        print("set_focal_length called:", length)
        return self



from enum import Enum
from typing import Tuple


class MaterialProperty:
    r: float
    g: float
    b: float
    a: float
    reflectivity: float
    roughness: float

    def __init__(
        self,
        color: Tuple[float, float, float, float],
        reflectivity: float,
        roughness: float,
    ):
        self.r, self.g, self.b, self.a = color
        self.reflectivity = reflectivity
        self.roughness = roughness


class PresetMaterial(Enum):
    black = MaterialProperty((0, 0, 0, 1.0), 0.0, 1.0)
    white = MaterialProperty((255, 255, 255, 1.0), 0.0, 1.0)
    red = MaterialProperty((255, 0, 0, 1.0), 0.0, 1.0)
    green = MaterialProperty((0, 255, 0, 1.0), 0.0, 1.0)
    blue = MaterialProperty((0, 0, 255, 1.0), 0.0, 1.0)
    yellow = MaterialProperty((255, 255, 0, 1.0), 0.0, 1.0)
    cyan = MaterialProperty((0, 255, 255, 1.0), 0.0, 1.0)
    magenta = MaterialProperty((255, 0, 255, 1.0), 0.0, 1.0)
    silver = MaterialProperty((192, 192, 192, 1.0), 0.0, 1.0)
    gray = MaterialProperty((128, 128, 128, 1.0), 0.0, 1.0)
    maroon = MaterialProperty((128, 0, 0, 1.0), 0.0, 1.0)
    olive = MaterialProperty((128, 128, 0, 1.0), 0.0, 1.0)
    green2 = MaterialProperty((0, 128, 0, 1.0), 0.0, 1.0)
    purple = MaterialProperty((128, 0, 128, 1.0), 0.0, 1.0)
    teal = MaterialProperty((0, 128, 128, 1.0), 0.0, 1.0)
    navy = MaterialProperty((0, 0, 128, 1.0), 0.0, 1.0)

    @property
    def color(self):
        r = self.value.r
        g = self.value.g
        b = self.value.b
        a = self.value.a
        return (r, g, b, a)

    @property
    def reflectivity(self):
        return self.value.reflectivity

    @property
    def roughness(self):
        return self.value.roughness



# THIS IS AN AUTO-GENERATE FILE.
# DO NOT EDIT MANUALLY.
# Please run development/capabilities_json_to_python/capabilities_to_py.sh to generate this file.

from typing import Optional
from abc import ABCMeta, abstractmethod

from codetocad.codetocad_types import *
from codetocad.utilities import *
from codetocad.core import *
from codetocad.enums import *


from codetocad.interfaces import ProjectableInterface

from . import EntityInterface

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from . import EntityInterface


class VertexInterface(EntityInterface, ProjectableInterface, metaclass=ABCMeta):
    """A single point in space, or a control point."""

    location: "Point"
    parent_entity: Optional[EntityOrItsName] = None

    @abstractmethod
    def __init__(
        self,
        location: "Point",
        name: str,
        parent_entity: Optional[EntityOrItsName] = None,
        description: Optional[str] = None,
        native_instance=None,
    ):
        super().__init__(
            name=name, description=description, native_instance=native_instance
        )
        self.location = location
        self.parent_entity = parent_entity
        self.name = name
        self.description = description
        self.native_instance = native_instance

    @abstractmethod
    def get_control_points(self, parameter="") -> "list[VertexInterface]":
        """
        Get a vertex's curve control points. This may not be applicable in several situations.
        """

        print(
            "get_control_points is called in an abstract method. Please override this method."
        )
        raise NotImplementedError()
